package controllers

import play.api._
import play.api.Play._
import play.api.mvc._
import play.api.libs.json._
import play.api.libs.json.Json._

import play.api.data.Forms._
import play.api.data._

import events._
import sourcing._
import models._
import models.sourcing._


object Application extends Controller {

	val loginForm = Form( 
		"userName" -> nonEmptyText 
	)

	val createForm = Form( 
		mapping(
			"userName" -> nonEmptyText,
			"firstName" -> nonEmptyText,
			"lastName" -> nonEmptyText,
			"age" -> number 
		){
			 case (u, f, l, a) => User(u, f, l, a)
		}{
			 case (u:User) => Some(u.userName, u.firstName, u.lastName, u.age)
		}
	)

	def index = Action { implicit request =>
		request.session.get("userName").map { _ => Redirect( routes.Application.myPage() ) }.getOrElse { Redirect( routes.Application.login() ) } 
	}

	def myPage = Action { implicit request => {
			//val userId = request.session.get("userId").get 
			//val withFriendsUsers = ModelSourcing.objects().current.values.filter(u => u.isInstanceOf[User] && u.id != userId).toSeq.asInstanceOf[Seq[User]]
			Ok(views.html.mypage())
		}
	}

	def logout = Action { implicit request =>
		Ok(views.html.login(loginForm)).withNewSession
	}

	def login = Action { implicit request =>
		Ok(views.html.login(loginForm))
	}

	def submit = Action { implicit request =>
		loginForm.bindFromRequest.fold (
		    formWithErrors => BadRequest( "You need to pass a 'usernName' value!" ),
		    { userName =>
				ModelSourcing.objects().current.find(o => o._2.isInstanceOf[User] && o._2.asInstanceOf[User].userName == userName) match {
					case Some((i, u:User)) => Redirect( routes.Application.myPage ).withSession("userName" -> userName, "userId" -> i)
					case _ =>  Unauthorized("Unknown " + userName)
				}
		    }
		)		
	}

	def create = Action { implicit request =>
	  Ok(views.html.create(createForm))
	}

	def register = Action { implicit request =>
	  createForm.bindFromRequest.fold (
	  	    formWithErrors => BadRequest( "Missing stuffs" ),
	  	    user => {
	  	        //check unicity

				ModelSourcing.transaction {
					events.Created(user)
				}

				Redirect( routes.Application.login() )
  		    }
	  	)			
	}

}