package controllers

import play.api._
import play.api.mvc._
import play.api.data.Forms._
import play.api.data._
/** Uncomment the following lines as needed **/
/**
import play.api.Play.current
import play.api.libs._
import play.api.libs.iteratee._
import play.api.libs.concurrent._
import java.util.concurrent._
import scala.concurrent.stm._
import akka.util.duration._
import play.api.cache._
**/

import events._
import sourcing._
import models._
import models.sourcing._

import play.api.libs.json._
import play.api.libs.json.Json._

import java.util.UUID.randomUUID
import scala.util.Random.nextInt

import scalaz._
import Scalaz._

object Users extends Controller {

	def random(nb:Int) = Action { 
      ModelSourcing.transaction {
         (1 to nb).map {_ => events.Created(User(randomUUID.toString, randomUUID.toString, randomUUID.toString, nextInt))} :_*
      }

		Ok("created " + nb)
	}

   def all = Action {
      Ok(toJson(ModelSourcing.objects().current.values.filter(_.isInstanceOf[User]).asInstanceOf[Iterable[User]].toSeq))
   }

   def get(id:String) = Action {
      ModelSourcing.objects().current.find(_._1 == id) match {
         case None =>  NotFound(toJson(Map("user" -> id)))
         case Some((i, u:User)) => Ok(toJson(u))
         case Some(x) => BadRequest(toJson(Map("error" -> "Bad Type")))
      }
   }

   def friends(id:String) = Action { implicit request => 
      Ok(views.html.friends(ModelSourcing.objects().current
            .find(_._1 == id)
            .map(u => User.structure.friends(u._2.asInstanceOf[User]))
            .getOrElse(Seq())
         )
      )
   }

   def addFriends(id:String) = Action { implicit request => {
         val user = ModelSourcing.objects().current(id).asInstanceOf[User]
         val ufs = user.friends
         val nonDirectFriends = 
            ModelSourcing.objects().current.values.view.
               filter(u => u.isInstanceOf[User] && u.id != id && !ufs.exists(_ == u.id)).
               toSeq.asInstanceOf[Seq[User]]
         Ok(views.html.addFriends(nonDirectFriends))
      }
   }

   val friendForm = Form( "friend" -> nonEmptyText )

   def addFriend(id:String) = Action { implicit request =>
      friendForm.bindFromRequest.fold (
          formWithErrors => BadRequest( "boom... cannot add friend" ),
          { friendId => {
               ModelSourcing.get(friendId) match {
                  case None => BadRequest("bad id")
                  case Some(u:User) => {
                     ModelSourcing.transaction(AddFriend(id, friendId))
                     Ok(friendId)
                  }
                  case _ => BadRequest("bad instance")
               }
            }
          }
      )      
   }

   def removeFriend(id:String, friend:String) = Action { implicit request =>
      ModelSourcing.get(id) match {
         case None => BadRequest("bad id")
         case Some(u:User) => {
            u.friends.find(_ == friend) match {
               case None => BadRequest("bad friend")
               case Some(_) => {
                  ModelSourcing.transaction(RemoveFriend(id, friend))
                  Ok(friend)
               }
            }
         }
         case _ => BadRequest("bad instance")
      }      
   }

   def withFriends = Action {
      Ok( toJson(ModelSourcing.objects().current.view.filter({
            case (s, (u:User)) => !User.structure.friends(u).isEmpty
            case _ => false
         }).map(_._2).toSeq.asInstanceOf[Seq[User]]))
   }






   def test(newUN:String, newFS:String,newLS:String,newA:Int) = Action {
      import User.structure._

      val user = User(randomUUID.toString, randomUUID.toString, randomUUID.toString, nextInt)

      ModelSourcing.inTransaction {
         for {
            state1 <- writer(Seq(events.Created(user):Event), user);
            state2 <- esUserName.set(state1, newUN)
         } yield state2
      }

      val friend = User(randomUUID.toString, randomUUID.toString, randomUUID.toString, nextInt)

      ModelSourcing.transaction(events.Created(friend))


      ModelSourcing.inTransaction {

         for {
            state1 <- esFirstName.set(ModelSourcing.objects().current(user.id).asInstanceOf[User], newFS);
            state2 <- esLastName.set(state1, newLS);
            state3 <- esAge.set(state2, newA);
            state4 <- esAddFriend(friend)(state3)._1;
            f      <- state4.map {_ => friend} // => inject friend in the transaction
            state5 <- esLastName.set(f, "myFriend");
            state6 <- state5.flatMap { _ => events.Created(Post(user.id, None, "my first post"))} //create and inject
         } yield state6


      }

		Ok(ModelSourcing.objects().events.reverse.map{_.toString}.mkString("\n"))
   }

}