package models

import play.api.libs.json._

import scalaz._
import scalaz.Scalaz._
import events._

import _root_.sourcing.ModelSourcing

case class Post(
	by:String, 
	replyTo:Option[String], 
	content:String,
	id:String = java.util.UUID.randomUUID.toString) 
	extends Model {	
}

object Post {
	
	implicit object PostFormat extends Format[Post] {

		def reads(js:JsValue) = if (js.isInstanceOf[JsObject]) {
			Post(
				(js \ "by").as[String],
				(js \ "replyTo").as[Option[String]],
				(js \ "content").as[String],
				(js \ "id").as[String]
			)
		} else {
			null//BAD really bad... 
		}

		def writes(post:Post) = JsObject(Seq(
			"by" -> JsString(post.by), 
			"replyTo" -> post.replyTo.map{r => JsString(r)}.getOrElse(JsNull), 
			"content" -> JsString(post.content), 
			"id" -> JsString(post.id)
		))

	}

	object structure {
		import sourcing._

		implicit def postToInit(post:Post):Event = Init(post)

		def by:Lens[Post,String] = Lens(_.by, (p,s)=>p.copy(by=s))
		def replyTo:Lens[Post,Option[String]] = Lens(_.replyTo, (p,s)=>p.copy(replyTo=s))
		def content:Lens[Post,String] = Lens(_.content, (p,s)=>p.copy(content=s))

		def esBy = eventSoucedFieldLens(by)(_.id, SetBy.apply)
		def esReplyTo = eventSoucedFieldLens(replyTo)(_.id, SetReplyTo.apply)
		def esContent = eventSoucedFieldLens(content)(_.id, SetContent.apply)
	}

}