package models

import play.api.libs.json._

import scalaz._
import scalaz.Scalaz._
import events._

import _root_.sourcing.ModelSourcing

case class User(
	userName:String, 
	firstName:String, 
	lastName:String, 
	age:Int,
	friends:Seq[String] = Seq(), 
	id:String = java.util.UUID.randomUUID.toString) 
	extends Model {	
}

object User {
	
	implicit object UserFormat extends Format[User] {

		def reads(js:JsValue) = if (js.isInstanceOf[JsObject]) {
			User(
				(js \ "userName").as[String],
				(js \ "firstName").as[String],
				(js \ "lastName").as[String],
				(js \ "age").as[Int],
				Seq(), 
				(js \ "id").as[String]
			)
		} else {
			null//BAD really bad... 
		}

		def writes(user:User) = JsObject(Seq(
			"userName" -> JsString(user.userName), 
			"firstName" -> JsString(user.firstName), 
			"lastName" -> JsString(user.lastName), 
			"age" -> JsNumber(user.age), 
			"id" -> JsString(user.id)
		))

	}

	object structure {
		import sourcing._

		implicit def userToInit(user:User):Event = Init(user)

		def userName:Lens[User,String] = Lens(_.userName, (u,s)=>u.copy(userName=s))
		def firstName:Lens[User,String] = Lens(_.firstName, (u,s)=>u.copy(firstName=s))
		def lastName:Lens[User,String] = Lens(_.lastName, (u,s)=>u.copy(lastName=s))
		def age:Lens[User,Int] = Lens(_.age, (u,i)=>u.copy(age=i))
		def _friends:Lens[User,Seq[String]] = Lens(_.friends, (u,fs)=>u.copy(friends=fs))
		def _addFriend(target:User, newFriend:String):User = _friends.mod(target, fs => newFriend +: fs)
		def _removeFriend(target:User, friend:String):User = _friends.mod(target, fs => fs.filter(_ != friend))
		def friends:Lens[User,Seq[User]] = 
			_friends.xmap { s =>
				s.map{ModelSourcing.objects().current(_).asInstanceOf[User]}
			} { s => 
				s.map{_.id}
			}
		def addFriend(target:User, newFriend:User):User = _friends.mod(target, fs => newFriend.id +: fs)

		def esUserName = eventSoucedFieldLens(userName)(_.id, SetUserName.apply)
		def esFirstName = eventSoucedFieldLens(firstName)(_.id, SetFirstName.apply)
		def esLastName = eventSoucedFieldLens(lastName)(_.id, SetLastName.apply)
		def esAge = eventSoucedFieldLens(age)(_.id, SetAge.apply)
		def esFriend = eventSoucedFieldLens(friends)(_.id, SetFriends.apply)
		def esAddFriend(friend:User) = stateWriter((u:User) => u.id, AddFriend.apply)(friend.id)
		def esRemoveFriend(friend:User) = stateWriter((u:User) => u.id, RemoveFriend.apply)(friend.id)
	}

}