package sourcing

import models._
import events._

import scalaz._
import Scalaz._

import scala.concurrent.stm.Ref

trait Sourcing {this:ModelSourcing with Sourcing =>
	import ModelSourcing._

	def current:Map[String, Model]
	def events:Seq[Event]

	def _apply:PartialFunction[Event, ModelSourcing] = {
		case _ => this //ignored
	}

	def apply(event:Event):ModelSourcing = _apply(event)

	def seq(es:Event*) = {
		es.foldLeft(this)((ms, e) => ms apply e)
	}

	private[sourcing] def applyEvent(id:String, e:Event) = current.find(_._1==id) map { iu => e.apply(iu._2) }

	private[sourcing] def updateModel = 
		(event:Event) => 
			(model:Model) => 
				eventsLens.mod(
					objectsLens.mod(this, _ + (model.id -> model)), 
					event +: _
				)
}

case class ModelSourcing(current:Map[String, Model] = Map(), events:Seq[Event] = Seq()) 
	extends UserSourcing
	with PostSourcing 
	with Sourcing {

	override def _apply:PartialFunction[Event, ModelSourcing] = super._apply
}

object ModelSourcing {
	val objects = Ref(ModelSourcing()).single

	def objectsLens:Lens[ModelSourcing, Map[String, Model]] = Lens(_.current, (os, m) => os.copy(current=m))
	def eventsLens:Lens[ModelSourcing, Seq[Event]] = Lens(_.events, (us, m) => us.copy(events=m))

	def transaction(events:Event*) = objects.transform(_ seq (events:_*))

	def inTransaction(block: =>Writer[Seq[Event], _]) = transaction(block.written:_*)

	def restore(previous:ModelSourcing, events:Event*) = objects.transform(_ => previous seq (events:_*))

	def get(id:String) = objects().current.get(id)
}