import com.excilys.ebi.gatling.core.Predef._
import com.excilys.ebi.gatling.http.Predef._

import play.api.test._
import play.api.test.Helpers.running

import play.api.libs.Files._

class CreateLoginLogoutRelogin() extends PlaySimulation {
    
    val port = 3333

    implicit val application = FakeApplication()

    def apply = {
      val baseUrl = "http://localhost:3333"

      val headers_1 = Map(
        "Accept" -> "text/html,application/xhtml+xml,application/xml,application/json;q=0.9,*/*;q=0.8",
        "Accept-Charset" -> "ISO-8859-1,utf-8;q=0.7,*;q=0.7"
      )

      val form = controllers.Application.loginForm
      val createForm = controllers.Application.createForm

      val users = (1 to 100) map {i => models.User("user"+i, "firstName"+i, "lastName"+i, i)}
      val allUsers = Global.testUsers ++ users
      
      val createFeed = simpleFeeder("createUsers", users map {u => createForm.fill(u).data}:_*) 
      val usersFeed = simpleFeeder("createdUsers",  users.map(u => form.fill(u.userName).data):_*)
      val allFeed = simpleFeeder("allUsers",  allUsers.map(u => form.fill(u.userName).data):_*)

      val scn = scenario("createLoginLogoutRelogin")
        .exec(
          http("login-page")
            .get(controllers.routes.Application.login.url)
            .headers(headers_1)
            .check(status.is(200))
        ).pause(5)
        .exec(
          http("create-page")
            .get(controllers.routes.Application.create.url)
            .headers(headers_1)
            .check(status.is(200))
        ).pause(30)
        .feed(createFeed)
        .exec(
          http("submit-create")
            .post(controllers.routes.Application.register.url)
            /*TODO boilerplate that could be removed... using Form info*/
            .param("userName", "${userName}")
            .param("firstName", "${firstName}")
            .param("lastName", "${lastName}")
            .param("age", "${age}")
            .headers(headers_1)
            .check(status.is(200))
        )
        .pause(10)
        .feed(usersFeed)
        .exec(
          http("loginCreated")
            .post(controllers.routes.Application.submit.url)
            .param("userName", "${userName}")
            .headers(headers_1)
            .check(status.is(200))
        )
        .pause(5, 25) //random pause
        .exec(
          http("logout")
            .get(controllers.routes.Application.logout.url)
            .headers(headers_1)
            .check(status.is(200))
        )
        .exec(
          http("login-page")
            .get(controllers.routes.Application.login.url)
            .headers(headers_1)
            .check(status.is(200))
        )
        .pause(10)
        .feed(usersFeed)
        .exec(
          http("try-login")
            .post(controllers.routes.Application.submit.url)
            .param("userName", "${userName}")
            .headers(headers_1)
            .check(status.is(200))
        )

      val httpConf = httpConfig.baseURL(baseUrl)

      Seq(
        scn.configure users 100 ramp 1 protocolConfig httpConf
      )
    }
}