import sbt._
import Keys._
import PlayProject._

/*import all stuff (keys, ...) defined in the sbt plugin*/
import GatlingPlugin._

object ApplicationBuild extends Build {
    val SNAPSHOT = "-SNAPSHOT"

    val appName         = "gatling-play2-sample"
    val appVersion      = "0.0.1-SNAPSHOT"

    val scalazVersionNumber = "6.0.4"
    val scalaz = "org.scalaz" %% "scalaz-core" % scalazVersionNumber withSources

  /* LOCAL MAVEN REPO */
  val localMavenRepo = "Local Maven Repository" at file(Path.userHome.absolutePath+"/.m2/repository").toURI.toURL.toString

  val gatSbtTestVersion = "0.0.1-SNAPSHOT"

  val cloudbees = "https://repository-andy-petrella.forge.cloudbees.com/"
  val cloudbeesRepo = gatSbtTestVersion match {
      case x if x.endsWith(SNAPSHOT) => x.toLowerCase at cloudbees + "snapshot" + "/"
      case x => x.toLowerCase at cloudbees + "release" + "/"
  }


  val appDependencies = Seq(
      scalaz,

     "be.nextlab" %% "gatling-play2-plugin" % gatSbtTestVersion % "gatling-test"
  )

    // Only compile the bootstrap bootstrap.less file and any other *.less file in the stylesheets directory
    def customLessEntryPoints(base: File): PathFinder = (
        (base / "app" / "assets" / "stylesheets" / "bootstrap" * "bootstrap.less") +++
        (base / "app" / "assets" / "stylesheets" * "*.less")
    )

    //Project.defaultSettings ++ 
    lazy val allSettings = gatlingSettings ++ 
      Seq(
        lessEntryPoints <<= baseDirectory(customLessEntryPoints),

        resolvers ++= Seq(cloudbeesRepo),

        javacOptions += "-Xlint:unchecked",
        libraryDependencies ++= appDependencies,

        /*todo still needed...*/
        commands ++= Seq(gatlingTakatak),

        /*todo could be automatically done for play project...*/
        galtlingConfFile in GatlingTest <<= baseDirectory { _ / "gatling-test" / "conf" / "galing.conf" },
        scalaSource in GatlingTest <<= baseDirectory { _ / "gatling-test" / "simulations" },
        sourceDirectories in GatlingTest <+= baseDirectory { _ / "gatling-test" / "simulations" }
      )


    val main = PlayProject(appName, appVersion, mainLang = SCALA).configs(GatlingTest).settings(allSettings: _*)

}
